/* -*- c++ -*- */
/*
 * Copyright 2016 The Aerospace Corporation.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_GOESSDR_MPC_DELAY_H
#define INCLUDED_GOESSDR_MPC_DELAY_H

#include <goessdr/api.h>
#include <gnuradio/block.h>

namespace gr {
  namespace goessdr {

    // below if a copy of the gr::blocks::delay code with minor changes
    /*!
     * \brief delay the input by a certain number of samples
     * \ingroup goessdr
     *
     * Message passing controlled (mpc) acts just like gr::blocks::delay but 
     * provides the message port set_dly which accepts a single (long) argument
     * as the delay.
     */
    class GOESSDR_API mpc_delay : virtual public block
    {
    public:
      // gr::goessdr::mpc_delay::sptr
      typedef boost::shared_ptr<mpc_delay> sptr;

      /*!
       * \brief Make a delay block.
       * \param itemsize size of the data items.
       * \param delay number of samples to delay stream (>= 0).
       */
      static sptr make(size_t itemsize, int delay);

      virtual int dly() const = 0;

      /*!
       * \brief Reset the delay.
       * \param d change the delay value. This can be a positive or
       * negative value.
       */
      virtual void set_dly(int d) = 0;
    };

  } // namespace goessdr
} // namespace gr

#endif /* INCLUDED_GOESSDR_MPC_DELAY_H */

