# -*- utf-8 -*-

import ctypes

# #defines from libaec.h

AEC_OK                      =   0
AEC_CONF_ERROR              =  -1
AEC_STREAM_ERROR            =  -2
AEC_DATA_ERROR              =  -3
AEC_MEM_ERROR               =  -4

# #defines from szlib.h
SZ_ALLOW_K13_OPTION_MASK    =   1
SZ_CHIP_OPTION_MASK         =   2
SZ_EC_OPTION_MASK           =   4
SZ_LSB_OPTION_MASK          =   8
SZ_MSB_OPTION_MASK          =  16
SZ_NN_OPTION_MASK           =  32
SZ_RAW_OPTION_MASK          = 128

SZ_OK = AEC_OK              =   0
SZ_OUTBUFF_FULL             =   2

SZ_NO_ENCODER_ERROR         =  -1
SZ_PARAM_ERROR              = AEC_CONF_ERROR
SZ_MEM_ERROR                = AEC_MEM_ERROR
SZ_MAX_PIXELS_PER_BLOCK     =  32
SZ_MAX_BLOCKS_PER_SCANLINE  = 128
SZ_MAX_PIXELS_PER_SCANLINE  = SZ_MAX_BLOCKS_PER_SCANLINE * SZ_MAX_PIXELS_PER_BLOCK

# parameter structure in libaec.h
class aec_stream_struct(ctypes.Structure):
    _fields_ = [("next_in", ctypes.POINTER(ctypes.c_ubyte)),
                ("avail_in", ctypes.c_size_t),
                ("total_in", ctypes.c_size_t),
                ("next_out", ctypes.POINTER(ctypes.c_ubyte)),
                ("avail_out", ctypes.c_size_t),
                ("total_out", ctypes.c_size_t),
                ("bits_per_sample", ctypes.c_uint),
                ("block_size", ctypes.c_uint),
                ("rsi", ctypes.c_uint),
                ("flags", ctypes.c_uint),
                ("internal_state", ctypes.c_void_p)
               ]

# parameter structure in szlib.h
class sz_com_struct(ctypes.Structure):
    _fields_ = [("options_mask", ctypes.c_int),
                ("bits_per_pixel", ctypes.c_int),
                ("pixels_per_block", ctypes.c_int),
                ("pixels_per_scanline", ctypes.c_int)
               ]

# library and function prototype
LIBNAME = 'libsz.so'
libsz = ctypes.CDLL(LIBNAME)
_SZ_BufftoBuffDecompress = libsz.SZ_BufftoBuffDecompress
_SZ_BufftoBuffDecompress.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_size_t),
                                     ctypes.c_void_p, ctypes.c_size_t,
                                     ctypes.POINTER(sz_com_struct)]
_SZ_BufftoBuffDecompress.restype = ctypes.c_int

MAX_COMPRESSION_RATIO = 32
def decompress(src, flags, bits_per_pxl, pxls_per_blk, pxl_per_scl):
    """Return the decompressed string

    :param compressed: string of compressed bytes
    :param mask: or'd set of mask options
    :param bits_per_pxl: integer bits per pixel
    :param pxl_per_blk: integer pixels per encoded block
    :param pxl_per_scnln: integer pixels per scanline (i.e. image columns)
    :returns: string of decompressed bytes
    :raises: ValueError, RuntimeError
    """

    params = sz_com_struct(flags, bits_per_pxl, pxls_per_blk, pxl_per_scl)
    srclen = ctypes.c_size_t(len(src))
    dstlen = ctypes.c_size_t(pxl_per_scl)

    dst = ctypes.create_string_buffer(dstlen.value)
    rv = _SZ_BufftoBuffDecompress(dst, ctypes.byref(dstlen),
                                      src, srclen,
                                      ctypes.byref(params))
    if rv != 0:
        print("Error %d in rice decompression!" % rv)


    return dst[:dstlen.value]

# quick test code
import sys
if __name__ == '__main__':
    with open(sys.argv[1], 'rb') as f:
        src = f.read()
    flgs, bpp, ppb, bps = tuple(int(x) for x in sys.argv[2:])
    dst = decompress(src, flgs, bpp, ppb, bps)
    sys.stdout.write(dst)
