from AbstractHeader import AbstractHeader

##
# @brief Data filename Header class.
# @details Contains the filename
# @warning Not in the LRIT spec, but in the C++ implementation and shows up
# in real life
# @author Miguel Dominguez
# @date 2016
class DataFilenameHeader(AbstractHeader):

   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 132
   @staticmethod
   def headerIndex():
      return 132

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.filename = "".join(map(chr,data[3:self.headerRecordLength]))
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Data Filename Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Annotation: {0}\n".format(self.filename)
      return output
