from AbstractHeader import AbstractHeader

##
# @brief Image Structure header class.
# @details File encoding information for images
# @author Miguel Dominguez
# @date 2016
class ImageStructureHeader(AbstractHeader):
   
   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 1
   @staticmethod
   def headerIndex():
      return 1

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.bitsPerPixel = data[3]
      self.columnCount = (data[4] << self.BYTE_SHIFT) | data[5]
      self.lineCount = (data[6] << self.BYTE_SHIFT) | data[7]
      self.compressionFlag = data[8]
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Image Structure Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Bits Per Pixel: {0}\n".format(self.bitsPerPixel)
      output += "Column Count: {0}\n".format(self.columnCount)
      output += "Line Count: {0}\n".format(self.lineCount)
      output += "Compression Flag: {0}\n".format(self.compressionFlag)
      return output
